import 'dart:ui';
import 'dart:math';


import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:yuanjs_easy_chart/tools/data.dart';
import 'package:yuanjs_easy_chart/tools/monotonex.dart';

class EasyLinePainter extends CustomPainter {
  final List<ChartData> beans; //待展示数据
  final String xName;
  final String yName;
  final Color storkColor;
  final Color xyColor;

  double startX, endX, startY, endY;

  bool isCurve;

  EasyLinePainter(this.beans, this.xName, this.yName, this.isCurve,
      this.storkColor, this.xyColor);

  final double innerPadding = 10.0;
  final double outerPadding = 20.0;

  @override
  void paint(Canvas canvas, Size size) {
    _initSize(size);

    Paint xyPaint = Paint()
      ..color = this.xyColor
      ..strokeWidth = 1;

    Paint paint = Paint()
      ..isAntiAlias = true //抗锯齿
      ..strokeWidth = 2
      ..strokeCap = StrokeCap.round //折线连接处圆滑处理
      ..color = this.storkColor
      ..style = PaintingStyle.stroke;

    var length = this.beans.length;

    double allX = endX - startX;
    double allY = endY - startY;

    double minY = this.beans[0].yValue.toDouble();
    double maxY = this.beans[0].yValue.toDouble();
    for (var i = 0; i < length; i++) {
      if (this.beans[i].yValue < minY) {
        minY = this.beans[i].yValue.toDouble();
      }
      if (this.beans[i].yValue > maxY) {
        maxY = this.beans[i].yValue.toDouble();
      }
    }

    print('maxY' + maxY.toString());
    print('minY' + minY.toString());
    print('allX' + allX.toString());
    print('allY' + allY.toString());

    List<Offset> points = [];
    List<Point> p_s = [];
    for (var i = 0; i < length; i++) {
      double temp = (this.beans[i].yValue - minY) / (allY);

      double x = startX + ((allX / (length + 1)) * (i + 1));
      double y = endY - 50 - temp * allY;
      points.add(Offset(x, y));
      p_s.add(Point(x, y));
      canvas.drawCircle(Offset(x, y), 3, paint);
    }
    //y轴文字
    var a = '1';
    while (a.length < maxY.floor().toString().length) {
      a = a + '0';
    }

    int yStep = (maxY / int.parse(a)).floor() + 1;

    for (var i = 0; i < yStep; i++) {
      double temp = ((i + 1) * int.parse(a) - minY) / (allY);

      canvas.drawLine(Offset(startX + 5, endY - 50 - temp * allY),
          Offset(startX + 10, endY - 50 - temp * allY), xyPaint);
      TextPainter(
          text: TextSpan(
              text: '${(i + 1) * int.parse(a)}',
              style: TextStyle(color: this.xyColor)),
          textDirection: TextDirection.ltr)
        ..layout()
        ..paint(canvas, Offset(startX - 15, endY - 50 - temp * allY));
    }

    canvas.drawLine(Offset(startX + innerPadding, startY + innerPadding),
        Offset(startX + innerPadding, endY - innerPadding), xyPaint); //y轴
    canvas.drawLine(Offset(startX + innerPadding, endY - innerPadding),
        Offset(endX - innerPadding, endY - innerPadding), xyPaint); //x轴
    TextPainter(
        text: TextSpan(text: '0', style: TextStyle(color: this.xyColor)),
        textDirection: TextDirection.ltr)
      ..layout()
      ..paint(canvas, Offset(startX, endY - innerPadding)); //原点字符

    TextPainter(
        text: TextSpan(
            text: '${this.beans[0].xName}',
            style: TextStyle(color: this.xyColor)),
        textDirection: TextDirection.ltr)
      ..layout()
      ..paint(canvas, Offset(points[0].dx - 10, endY - innerPadding));

    canvas.drawLine(Offset(points[0].dx, endY - innerPadding),
        Offset(points[0].dx, endY - innerPadding - 5), xyPaint);
    //x轴数值标记
    for (var i = 1; i < points.length; i++) {
      TextPainter(
          text: TextSpan(
              text: '${this.beans[i].xName}',
              style: TextStyle(color: this.xyColor)),
          textDirection: TextDirection.ltr)
        ..layout()
        ..paint(canvas, Offset(points[i].dx - 10, endY - innerPadding)); //x轴文字
      canvas.drawLine(Offset(points[i].dx, endY - innerPadding),
          Offset(points[i].dx, endY - innerPadding - 5), xyPaint); //x轴数值标记
      if (!isCurve) {
        canvas.drawLine(points[i - 1], points[i], xyPaint);
      }
    }

    if (isCurve) {
      _drawSmoothLine(canvas, paint, p_s);
    }

    //x轴名称，y轴名称增加
    if (this.yName != null) {
      TextPainter(
          text: TextSpan(
              text: '${this.yName}', style: TextStyle(color: this.xyColor)),
          textDirection: TextDirection.ltr)
        ..layout()
        ..paint(canvas, Offset(startX, startY - innerPadding));
    }

    if (this.xName != null) {
      TextPainter(
          text: TextSpan(
              text: '${this.xName?.toString()}',
              style: TextStyle(color: this.xyColor)),
          textDirection: TextDirection.ltr)
        ..layout()
        ..paint(canvas, Offset(endX, endY - 2 * innerPadding));
    }
  }

  void _initSize(Size size) {
//    print('Size Width' + size.width.toString());
//    print('Size Height' + size.height.toString());
    startX = outerPadding;
    endX = size.width - outerPadding;

    startY = outerPadding;
    endY = size.height - outerPadding;
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }

//  void _drawSmoothLine(Canvas canvas, Paint paint, List<Point> points) {
//    var targetPoints = List<Point>();
//    targetPoints.add(points[0]);
//    targetPoints.addAll(points);
//    targetPoints.add(points[points.length - 1]);
//    final path = new Path();
//    for (int i = 1; i < targetPoints.length - 2; i++) {
//      path.moveTo(targetPoints[i].x.toDouble(), targetPoints[i].y.toDouble());
//      var controllerPoint1 = Point(
//        targetPoints[i].x + (targetPoints[i + 1].x - targetPoints[i - 1].x) / 4,
//        targetPoints[i].y + (targetPoints[i + 1].y - targetPoints[i - 1].y) / 4,
//      );
//      var controllerPoint2 = Point(
//        targetPoints[i + 1].x - (targetPoints[i + 2].x - targetPoints[i].x) / 4,
//        targetPoints[i + 1].y - (targetPoints[i + 2].y - targetPoints[i].y) / 4,
//      );
//      path.cubicTo(controllerPoint1.x, controllerPoint1.y, controllerPoint2.x,
//          controllerPoint2.y, targetPoints[i + 1].x, targetPoints[i + 1].y);
//    }
//    canvas.drawPath(path, paint);
//  }

  void _drawSmoothLine(Canvas canvas, Paint paint, List<Point> points) {
    var path = MonotoneX.initPath(
        points); //优化所在点，https://github.com/d3/d3-shape/blob/master/src/curve/monotone.js#L98
    canvas.drawPath(path, paint);
  }
}
